/* -*- Mode: C++; indent-tabs-mode: nil; tab-width: 4 -*-
 * -*- coding: utf-8 -*-
 *
 * Copyright (C) 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "media-key-block-shutcut.h"

class MediaKeyBlockShortcutPrivate : public QObject
{
    Q_OBJECT
public:
    explicit MediaKeyBlockShortcutPrivate(QObject* parent = nullptr);
    ~MediaKeyBlockShortcutPrivate(){USD_LOG(LOG_DEBUG,"析构..");}

Q_SIGNALS:
    void blockChanged(QMap<QString, QStringList> blockName);

private Q_SLOTS:
    void serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner);
private:
    void addBlock(const QString& service);
    void removeBlock(const QString& service);

private:
    QMap<QString, QStringList> m_blockList;
public:
};




MediaKeyBlockShortcutPrivate::MediaKeyBlockShortcutPrivate(QObject *parent)
    : QObject(parent)
{
#if (QT_VERSION >= QT_VERSION_CHECK(5, 15, 0))
    auto watcher = new QDBusServiceWatcher(QStringLiteral(USD_BLOCK_SETTINGS_DAEMON_SHORTCUT), QDBusConnection::sessionBus(), QDBusServiceWatcher::WatchForOwnerChange, this);
    connect(watcher, &QDBusServiceWatcher::serviceOwnerChanged, this, &MediaKeyBlockShortcutPrivate::serviceOwnerChanged);
#else
    connect(QDBusConnection::sessionBus().interface(), &QDBusConnectionInterface::serviceOwnerChanged, this, &MediaKeyBlockShortcutPrivate::serviceOwnerChanged);
#endif
    //获取列表
    QDBusPendingCall async = QDBusConnection::sessionBus().interface()->asyncCall(QStringLiteral("ListNames"));
    QDBusPendingCallWatcher *asyncWatcher = new QDBusPendingCallWatcher(async, this);
    connect(asyncWatcher, &QDBusPendingCallWatcher::finished, [=](QDBusPendingCallWatcher* watcher){
        QDBusPendingReply<QStringList> reply = *watcher;
        watcher->deleteLater();
        if (reply.isError()) {
            //error
            USD_LOG(LOG_DEBUG,"Error message:%s-->%s", reply.error().name().toLatin1().data(), reply.error().message().toLatin1().data());
        } else {
            for (const QString& serviceName : reply.value()) {
                if (serviceName.startsWith(QLatin1String(USD_BLOCK_SETTINGS_DAEMON_STARTWITH))) {
                    addBlock(serviceName);
                }
            }
        }
    });
}

void MediaKeyBlockShortcutPrivate::serviceOwnerChanged(const QString &name, const QString &oldOwner, const QString &newOwner)
{
    USD_LOG_SHOW_PARAMS(name.toLatin1().data());
    if (name.isEmpty()) {
        return;
    }

    if (!name.startsWith(USD_BLOCK_SETTINGS_DAEMON_STARTWITH)) {
        //过滤不符合mpris的seivice
        return;
    }
    if (!oldOwner.isEmpty()) {
        //dbus 注销 remove player
        removeBlock(name);
    }
    if (!newOwner.isEmpty()) {
        //dbus 注册 add player
        addBlock(name);
    }
}

void MediaKeyBlockShortcutPrivate::addBlock(const QString &service)
{
    QDBusInterface blockIter(service,
                             USD_BLOCK_SETTINGS_DAEMON_PATH,
                             USD_BLOCK_SETTINGS_DAEMON_INTERFEACE,
                            QDBusConnection::sessionBus());
    if (!blockIter.isValid()) {
        USD_LOG(LOG_DEBUG,"create interface error![%s]", service.toLatin1().data());
        return;
    }
     USD_LOG(LOG_DEBUG,"create interface success![%s]", service.toLatin1().data());

    QDBusPendingCall pendingCall = blockIter.asyncCall(USD_BLOCK_SETTINGS_DAEMON_METHOD);
    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(pendingCall,this);
    watcher->setProperty("serviceName", service);

    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this,watcher](){
        QDBusMessage msg  = watcher->reply();
        QString service = watcher->property("serviceName").toString();
        if (msg.type() == QDBusMessage::ReplyMessage) {
            if (msg.arguments().isEmpty() == false) {
                QStringList value = msg.arguments().takeFirst().toStringList();
                m_blockList.insert(service, value);
            }
        }
        Q_EMIT blockChanged(m_blockList);
        watcher->deleteLater();
    });
}

void MediaKeyBlockShortcutPrivate::removeBlock(const QString &service)
{
    m_blockList.remove(service);
    Q_EMIT blockChanged(m_blockList);
}


MediaKeyBlockShutcut::MediaKeyBlockShutcut(QObject *parent)
    : QObject(parent)
    , d(new MediaKeyBlockShortcutPrivate(this))
{
    connect(d, &MediaKeyBlockShortcutPrivate::blockChanged, this, &MediaKeyBlockShutcut::blockChanged);
    USD_LOG(LOG_DEBUG,".");
}

MediaKeyBlockShutcut::~MediaKeyBlockShutcut()
{
    USD_LOG(LOG_DEBUG,".");
}

#include "media-key-block-shutcut.moc"
